@@ -115,7 +115,7 @@ class EnvironmentVarGuard(MutableMapping):
                     del self._environ[k]
             else:
                 self._environ[k] = v
-        os.environ = self._environ
+        os.environ = self._environ  # noqa: B003
 
 
 @contextlib.contextmanager
@@ -124,6 +124,7 @@ def change_cwd(path, quiet=False):
     Return a context manager that changes the current working directory.
 
     Arguments:
+    ---------
       path: the directory to use as the temporary current working directory.
       quiet: if False (the default), the context manager raises an exception
         on error.  Otherwise, it issues only a warning and keeps the current
@@ -265,7 +266,7 @@ class _TestGlob:
                 try:
                     os.makedirs(base)
                     retry = 0
-                except Exception:
+                except Exception:  # noqa: PERF203
                     retry -= 1
         create_empty_file(filename)
 
@@ -298,7 +299,7 @@ class _TestGlob:
             try:
                 shutil.rmtree(cls.tempdir)
                 retry = 0
-            except Exception:
+            except Exception:  # noqa: PERF203
                 retry -= 1
 
     @staticmethod
@@ -509,7 +510,7 @@ class Testglob(_TestGlob):
             ] if not can_symlink() else [
                 ('',), ('aab',), ('aab', 'F'), ('a',), ('a', 'bcd'), ('a', 'bcd', 'EF'), ('a', 'bcd', 'efg'),
                 ('a', 'bcd', 'efg', 'ha'), ('a', 'D'), ('aaa',), ('aaa', 'zzzF'), ('EF',), ('ZZZ',),
-                ('sym1',), ('sym2',)
+                ('sym1',), ('sym2',), ('sym3',)
             ],
             glob.L
         ],
@@ -530,17 +531,13 @@ class Testglob(_TestGlob):
         Options(default_negate='sym3/EF'),
         [
             ('**', 'EF'),
-            [
-            ] if not can_symlink() else [
-            ],
+            [],
             glob.N | glob.L
         ],
 
         [
             ('**', 'EF'),
-            [
-            ] if not can_symlink() else [
-            ],
+            [],
             glob.N
         ],
 
@@ -552,7 +549,7 @@ class Testglob(_TestGlob):
             [
                 ('EF',), ('ZZZ',), ('',)
             ] if not can_symlink() else [
-                ('EF',), ('ZZZ',), ('',), ('sym1',), ('sym2',)
+                ('EF',), ('ZZZ',), ('',), ('sym1',), ('sym2',), ('sym3',)
             ],
             glob.N | glob.L
         ],
@@ -622,8 +619,8 @@ class Testglob(_TestGlob):
         Options(absolute=True, skip=sys.platform != "win32"),
         [('*:',), []],
         [('?:',), []],
-        [(r'\\\\?\\c:\\',), [('\\\\?\\c:\\',)]],
-        [(r'\\\\*\\*\\',), []],
+        [(R'\\\\?\\c:\\',), [('\\\\?\\c:\\',)]],
+        [(R'\\\\*\\*\\',), []],
         Options(absolute=False, skip=False),
 
         Options(skip=not can_symlink()),
@@ -658,7 +655,7 @@ class Testglob(_TestGlob):
             ]
         ],
         [
-            (r'*\\',),
+            (R'*\\',),
             [
                 ('aab', ''), ('aaa', ''), ('a', '')
             ] if not can_symlink() else [
@@ -674,7 +671,7 @@ class Testglob(_TestGlob):
         [('[a]',), [('a',)]],
         [('[!b]',), [('a',)]],
         [('[^b]',), [('a',)]],
-        [(r'@([\a]|\aaa)',), [('a',), ('aaa',)]],
+        [(R'@([\a]|\aaa)',), [('a',), ('aaa',)]],
 
         Options(absolute=True),
         # Test empty.
@@ -952,9 +949,7 @@ class Testglob(_TestGlob):
         Options(just_negative=True, default_negate='**'),
         [
             ('a*', '**'),
-            [
-            ] if not can_symlink() else [
-            ],
+            [],
             glob.N
         ],
         Options(just_negative=False, cwd_temp=False, absolute=False),
@@ -1000,6 +995,103 @@ class Testglob(_TestGlob):
         for file in glob.glob(b'!**/', flags=glob.N | glob.NEGATEALL | glob.G, root_dir=os.fsencode(self.tempdir)):
             self.assert_equal(os.path.isdir(file), False)
 
+    def test_magic_non_magic(self):
+        """Test logic when switching from magic to non-magic patterns."""
+
+        with change_cwd(self.tempdir):
+            self.assert_equal(sorted(glob.glob(['**/aab', 'dummy'], flags=glob.G)), ['aab',])
+
+    def test_non_magic_magic(self):
+        """Test logic when switching from non-magic to magic patterns."""
+
+        with change_cwd(self.tempdir):
+            self.assert_equal(sorted(glob.glob(['dummy', '**/aab'], flags=glob.G)), ['aab',])
+
+
+class TestGlobStarLong(_TestGlob):
+    """Test `***` cases."""
+
+    DEFAULT_FLAGS = glob.BRACE | glob.EXTGLOB | glob.GLOBSTARLONG | glob.MARK | glob.NOUNIQUE
+
+    cases = [
+        # Test `globstar` with `globstarlong` enabled
+        [
+            ('**', '*'),
+            [
+                ('aab',), ('aab', 'F'), ('a',), ('a', 'bcd'), ('a', 'bcd', 'EF'), ('a', 'bcd', 'efg'),
+                ('a', 'bcd', 'efg', 'ha'), ('a', 'D'), ('aaa',), ('aaa', 'zzzF'), ('EF',), ('ZZZ',)
+            ] if not can_symlink() else [
+                ('aab',), ('aab', 'F'), ('a',), ('a', 'bcd'), ('a', 'bcd', 'EF'), ('a', 'bcd', 'efg'),
+                ('a', 'bcd', 'efg', 'ha'), ('a', 'D'), ('aaa',), ('aaa', 'zzzF'), ('EF',), ('ZZZ',),
+                ('sym1',), ('sym2',), ('sym3',)
+            ]
+        ],
+
+        # Test `globstarlong`
+        [
+            ('***', '*'),
+            [
+                ('aab',), ('aab', 'F'), ('a',), ('a', 'bcd'), ('a', 'bcd', 'EF'), ('a', 'bcd', 'efg'),
+                ('a', 'bcd', 'efg', 'ha'), ('a', 'D'), ('aaa',), ('aaa', 'zzzF'), ('EF',), ('ZZZ',)
+            ] if not can_symlink() else [
+                ('aab',), ('aab', 'F'), ('a',), ('a', 'bcd'), ('a', 'bcd', 'EF'), ('a', 'bcd', 'efg'),
+                ('a', 'bcd', 'efg', 'ha'), ('a', 'D'), ('aaa',), ('aaa', 'zzzF'), ('EF',), ('ZZZ',),
+                ('sym1',), ('sym2',), ('sym3',), ('sym3', 'EF'), ('sym3', 'efg'), ('sym3', 'efg', 'ha')
+            ]
+        ],
+
+        # Enable `FOLLOW`. Should be no changes.
+        [
+            ('**', '*'),
+            [
+                ('aab',), ('aab', 'F'), ('a',), ('a', 'bcd'), ('a', 'bcd', 'EF'), ('a', 'bcd', 'efg'),
+                ('a', 'bcd', 'efg', 'ha'), ('a', 'D'), ('aaa',), ('aaa', 'zzzF'), ('EF',), ('ZZZ',)
+            ] if not can_symlink() else [
+                ('aab',), ('aab', 'F'), ('a',), ('a', 'bcd'), ('a', 'bcd', 'EF'), ('a', 'bcd', 'efg'),
+                ('a', 'bcd', 'efg', 'ha'), ('a', 'D'), ('aaa',), ('aaa', 'zzzF'), ('EF',), ('ZZZ',),
+                ('sym1',), ('sym2',), ('sym3',)
+            ],
+            glob.L
+        ],
+        [
+            ('***', '*'),
+            [
+                ('aab',), ('aab', 'F'), ('a',), ('a', 'bcd'), ('a', 'bcd', 'EF'), ('a', 'bcd', 'efg'),
+                ('a', 'bcd', 'efg', 'ha'), ('a', 'D'), ('aaa',), ('aaa', 'zzzF'), ('EF',), ('ZZZ',)
+            ] if not can_symlink() else [
+                ('aab',), ('aab', 'F'), ('a',), ('a', 'bcd'), ('a', 'bcd', 'EF'), ('a', 'bcd', 'efg'),
+                ('a', 'bcd', 'efg', 'ha'), ('a', 'D'), ('aaa',), ('aaa', 'zzzF'), ('EF',), ('ZZZ',),
+                ('sym1',), ('sym2',), ('sym3',), ('sym3', 'EF'), ('sym3', 'efg'), ('sym3', 'efg', 'ha')
+            ],
+            glob.L
+        ]
+    ]
+
+    @classmethod
+    def setup_fs(cls):
+        """Setup file system."""
+
+        cls.mktemp('a', 'D')
+        cls.mktemp('aab', 'F')
+        cls.mktemp('.aa', 'G')
+        cls.mktemp('.bb', 'H')
+        cls.mktemp('aaa', 'zzzF')
+        cls.mktemp('ZZZ')
+        cls.mktemp('EF')
+        cls.mktemp('a', 'bcd', 'EF')
+        cls.mktemp('a', 'bcd', 'efg', 'ha')
+        cls.can_symlink = can_symlink()
+        if cls.can_symlink:
+            os.symlink(cls.norm('broken'), cls.norm('sym1'))
+            os.symlink('broken', cls.norm('sym2'))
+            os.symlink(os.path.join('a', 'bcd'), cls.norm('sym3'))
+
+    @pytest.mark.parametrize("case", cases)
+    def test_glob_cases(self, case):
+        """Test glob cases."""
+
+        self.eval_glob_cases(case)
+
 
 class TestGlobMarked(Testglob):
     """Test glob marked."""
@@ -1322,14 +1414,14 @@ class TestGlobCornerCase(_TestGlob):
         [('@(a/b)',), []],
         [('@(a[/]b)',), []],
         [('test[',), [('test[',)]],
-        [(r'a\/b',), [('a', 'b')]],
-        [(r'a[\/]b',), [('a[', ']b')]],
+        [(R'a\/b',), [('a', 'b')]],
+        [(R'a[\/]b',), [('a[', ']b')]],
 
         Options(skip=util.is_case_sensitive()),
         [('a[\\',), [('a[',)]],
         [('@(a[\\',), [('@(a[',)]],
-        [(r'a[\\',), [('a[', '')]],
-        [(r'@(a[\\',), [('@(a[', '')]],
+        [(R'a[\\',), [('a[', '')]],
+        [(R'@(a[\\',), [('@(a[', '')]],
         Options(skip=False)
     ]
 
@@ -1360,7 +1452,7 @@ class TestGlobCornerCaseMarked(Testglob):
 class TestGlobEscapes(unittest.TestCase):
     """Test escaping."""
 
-    def check_escape(self, arg, expected, raw=False, unix=None, raw_chars=True):
+    def check_escape(self, arg, expected, unix=None):
         """Verify escapes."""
 
         flags = 0
@@ -1369,119 +1461,68 @@ class TestGlobEscapes(unittest.TestCase):
         elif unix is True:
             flags = glob.FORCEUNIX
 
-        if raw:
-            self.assertEqual(glob.raw_escape(arg, unix=unix, raw_chars=raw_chars), expected)
-            self.assertEqual(glob.raw_escape(os.fsencode(arg), unix=unix, raw_chars=raw_chars), os.fsencode(expected))
-            file = (util.norm_pattern(arg, False, True) if raw_chars else arg).replace('\\\\', '\\')
-            self.assertTrue(
-                glob.globmatch(
-                    file,
-                    glob.raw_escape(arg, unix=unix, raw_chars=raw_chars),
-                    flags=flags
-                )
+        self.assertEqual(glob.escape(arg, unix=unix), expected)
+        self.assertEqual(glob.escape(os.fsencode(arg), unix=unix), os.fsencode(expected))
+        self.assertTrue(
+            glob.globmatch(
+                arg,
+                glob.escape(arg, unix=unix),
+                flags=flags
             )
-        else:
-            self.assertEqual(glob.escape(arg, unix=unix), expected)
-            self.assertEqual(glob.escape(os.fsencode(arg), unix=unix), os.fsencode(expected))
-            self.assertTrue(
-                glob.globmatch(
-                    arg,
-                    glob.escape(arg, unix=unix),
-                    flags=flags
-                )
-            )
-
-    def test_raw_escape_deprecation(self):
-        """Test raw escape deprecation."""
-
-        with warnings.catch_warnings(record=True) as w:
-            warnings.simplefilter("always")
-
-            glob.raw_escape(r'test\\test')
-
-            self.assertTrue(len(w) == 1)
-            self.assertTrue(issubclass(w[-1].category, DeprecationWarning))
+        )
 
     def test_escape(self):
         """Test path escapes."""
 
         check = self.check_escape
         check('abc', 'abc')
-        check('[', r'\[')
-        check('?', r'\?')
-        check('*', r'\*')
-        check('[[_/*?*/_]]', r'\[\[_/\*\?\*/_\]\]')
-        check('/[[_/*?*/_]]/', r'/\[\[_/\*\?\*/_\]\]/')
-
-    def test_raw_escape(self):
-        """Test path escapes."""
-
-        check = self.check_escape
-        check(r'abc', 'abc', raw=True)
-        check(r'[', r'\[', raw=True)
-        check(r'?', r'\?', raw=True)
-        check(r'*', r'\*', raw=True)
-        check(r'[[_/*?*/_]]', r'\[\[_/\*\?\*/_\]\]', raw=True)
-        check(r'/[[_/*?*/_]]/', r'/\[\[_/\*\?\*/_\]\]/', raw=True)
-        check(r'\x3f', r'\?', raw=True)
-        check(r'\\\\[^what]\\name\\temp', r'\\\\[^what]\\name\\temp', raw=True, unix=False)
-        check('//[^what]/name/temp', r'//[^what]/name/temp', raw=True, unix=False)
-
-    def test_raw_escape_no_raw_chars(self):
-        """Test path escapes with no raw character translations."""
-
-        check = self.check_escape
-        check(r'abc', 'abc', raw=True, raw_chars=False)
-        check(r'[', r'\[', raw=True, raw_chars=False)
-        check(r'?', r'\?', raw=True, raw_chars=False)
-        check(r'*', r'\*', raw=True, raw_chars=False)
-        check(r'[[_/*?*/_]]', r'\[\[_/\*\?\*/_\]\]', raw=True, raw_chars=False)
-        check(r'/[[_/*?*/_]]/', r'/\[\[_/\*\?\*/_\]\]/', raw=True, raw_chars=False)
-        check(r'\x3f', r'\\x3f', raw=True, raw_chars=False)
-        check(r'\\\\[^what]\\name\\temp', r'\\\\[^what]\\name\\temp', raw=True, raw_chars=False, unix=False)
-        check('//[^what]/name/temp', r'//[^what]/name/temp', raw=True, raw_chars=False, unix=False)
+        check('[', R'\[')
+        check('?', R'\?')
+        check('*', R'\*')
+        check('[[_/*?*/_]]', R'\[\[_/\*\?\*/_\]\]')
+        check('/[[_/*?*/_]]/', R'/\[\[_/\*\?\*/_\]\]/')
 
     @unittest.skipUnless(sys.platform.startswith('win'), "Windows specific test")
     def test_escape_windows(self):
         """Test windows escapes."""
 
         check = self.check_escape
-        check('a:\\?', r'a:\\\?')
-        check('b:\\*', r'b:\\\*')
-        check('\\\\?\\c:\\?', r'\\\\?\\c:\\\?')
-        check('\\\\*\\*\\*', r'\\\\*\\*\\\*')
-        check('//?/c:/?', r'//?/c:/\?')
-        check('//*/*/*', r'//*/*/\*')
-        check('//[^what]/name/temp', r'//[^what]/name/temp')
+        check('a:\\?', R'a:\\\?')
+        check('b:\\*', R'b:\\\*')
+        check('\\\\?\\c:\\?', R'\\\\?\\c:\\\?')
+        check('\\\\*\\*\\*', R'\\\\*\\*\\\*')
+        check('//?/c:/?', R'//?/c:/\?')
+        check('//*/*/*', R'//*/*/\*')
+        check('//[^what]/name/temp', R'//[^what]/name/temp')
 
     def test_escape_forced_windows(self):
         """Test forced windows escapes."""
 
         check = self.check_escape
-        check('a:\\?', r'a:\\\?', unix=False)
-        check('b:\\*', r'b:\\\*', unix=False)
-        check('\\\\?\\c:\\?', r'\\\\?\\c:\\\?', unix=False)
-        check('\\\\*\\*\\*', r'\\\\*\\*\\\*', unix=False)
-        check('//?/c:/?', r'//?/c:/\?', unix=False)
-        check('//*/*/*', r'//*/*/\*', unix=False)
+        check('a:\\?', R'a:\\\?', unix=False)
+        check('b:\\*', R'b:\\\*', unix=False)
+        check('\\\\?\\c:\\?', R'\\\\?\\c:\\\?', unix=False)
+        check('\\\\*\\*\\*', R'\\\\*\\*\\\*', unix=False)
+        check('//?/c:/?', R'//?/c:/\?', unix=False)
+        check('//*/*/*', R'//*/*/\*', unix=False)
         check(
             '//./Volume{b75e2c83-0000-0000-0000-602f00000000}/temp',
-            r'//./Volume\{b75e2c83-0000-0000-0000-602f00000000\}/temp',
+            R'//./Volume\{b75e2c83-0000-0000-0000-602f00000000\}/temp',
             unix=False
         )
-        check('//[^what]/name/temp', r'//[^what]/name/temp', unix=False)
+        check('//[^what]/name/temp', R'//[^what]/name/temp', unix=False)
 
     def test_escape_forced_unix(self):
         """Test forced windows Unix."""
 
         check = self.check_escape
-        check('a:\\?', r'a:\\\?', unix=True)
-        check('b:\\*', r'b:\\\*', unix=True)
-        check('\\\\?\\c:\\?', r'\\\\\?\\c:\\\?', unix=True)
-        check('\\\\*\\*\\*', r'\\\\\*\\\*\\\*', unix=True)
-        check('//?/c:/?', r'//\?/c:/\?', unix=True)
-        check('//*/*/*', r'//\*/\*/\*', unix=True)
-        check('//[^what]/name/temp', r'//\[^what\]/name/temp', unix=True)
+        check('a:\\?', R'a:\\\?', unix=True)
+        check('b:\\*', R'b:\\\*', unix=True)
+        check('\\\\?\\c:\\?', R'\\\\\?\\c:\\\?', unix=True)
+        check('\\\\*\\*\\*', R'\\\\\*\\\*\\\*', unix=True)
+        check('//?/c:/?', R'//\?/c:/\?', unix=True)
+        check('//*/*/*', R'//\*/\*/\*', unix=True)
+        check('//[^what]/name/temp', R'//\[^what\]/name/temp', unix=True)
 
 
 @unittest.skipUnless(sys.platform.startswith('win'), "Windows specific test")
@@ -1611,7 +1652,7 @@ class TestGlobPaths(unittest.TestCase):
         self.assertTrue(len(results) > 0)
         self.assertTrue('\\' not in results)
 
-        results = glob.glob(r'\\*')
+        results = glob.glob(R'\\*')
         self.assertTrue(len(results) > 0)
         self.assertTrue('\\' not in results)
 
